# Generated by roxygen2: do not edit by hand

export(assess)
export(bias)
export(bootstrap)
export(encodedata)
export(estimate)
export(loadstate)
export(logratio)
export(plotdistribution)
export(plotestimates)
export(risk)
export(rmse)
export(sampleposterior)
export(savestate)
export(spatialgraph)
export(tanhloss)
export(train)
import(JuliaConnectoR)
importFrom(magrittr,"%>%")
importFrom(stats,reshape)
importFrom(utils,combn)
